# Code > Ref > Symbol 계층 구조는 어디서나 공통
# description 과 같은 파라미터에 필요한 추가 정보는 openapi schema의 규칙을 따라야 함
# 참조 https://swagger.io/docs/specification/data-models/keywords/
# https://www.learnjsonschema.com/2020-12/
node_list = [
    # Example: [EN1991-1-5] Range of uniform bridge temperature component
    {
        'id': "G1_NODE_1",
        'standard': 'EN1991-1-5',
        'title': 'Bridge deck types',
        'latex_symbol': r'Deck_types',
        'unit': '',
        'reference': '6.1.1(1)',
        'description': r"This section categorizes bridge decks into different types based on their material and structural design. The classification includes steel decks, composite decks, and concrete decks, each with distinct subtypes like steel box girders, concrete slabs, and concrete box girders.",
        'type': 'string',
        'default': 'steel box girder',
        'table': 'dropdown',
        'enum': [
            "steel box girder",
            "steel truss or plate girder",
            "Composite deck",
            "concrete slab",
            "concrete beam",
            "concrete box girder",
        ],
    },
    {
        'id': "G1_NODE_2",
        'standard': 'EN1991-1-5',
        'title': 'The initial bridge temperature',
        'latex_symbol': r'T_{0}',
        'unit': '°C',
        'reference': 'AnnexA.2(2)',
        'description': r"Initial temperature when the structural element is restrained",
        'type': 'number',
        'default' : 10.0,
    },
    {
        'id': 'G1_NODE_3',
        'standard': 'EN1991-1-5',
        'title': 'Return Period',
        'latex_symbol': r'p_{years}',
        'unit': 'years',
        'reference': '1.5.3, 1.5.4',
        'description': r"Annual probability of maximum (minimum) shade air temperature being exceeded",
        'type': 'number',
        'default': 50.0
    },
    {
        'id': "G1_NODE_4",
        'standard': 'EN1991-1-5',
        'title': 'Annual exceedance probability',
        'latex_symbol': r'p',
        'latex_equation': r'p = \frac{1}{p_{years}}',
        'unit': '',
        'reference': '1.6',
        'description': r"Annual probability of maximum (minimum) shade air temperature being exceeded (equivalent to a mean return period of 1/p years)",
        'type': 'number',
    },
    {
        'id': 'G1_NODE_5',
        'standard': 'EN1991-1-5',
        'title': 'Minimum shade air temperature (exceedance probability 0.02)',
        'latex_symbol': r'T_{min}',
        'unit': '°C',
        'reference': '6.1.3.2(1)P',
        'description': r"Minimum shade air temperature with an annual probability of being exceeded of 0.02 (equivalent to a mean return period of 50 years)",
        'type': 'number',
    },
    {
        'id': 'G1_NODE_6',
        'standard': 'EN1991-1-5',
        'title': 'Maximum shade air temperature (exceedance probability 0.02)',
        'latex_symbol': r'T_{max}',
        'unit': '°C',
        'reference': '6.1.3.2(1)P',
        'description': r"Maximum shade air temperature with an annual probability of being exceeded of 0.02 (equivalent to a mean return period of 50 years)",
        'type': 'number',
    },
    {
        'id': 'G1_NODE_7',
        'standard': 'EN1991-1-5',
        'title': 'Minimum shade air temperature with exceedance probability p',
        'latex_symbol': r'T_{min,p}',
        'latex_equation': r'T_{min,p} = T_{min} \cdot \left( k_{3} + k_{4} \cdot \ln \left[ - \ln \left( 1 - p \right) \right] \right)',
        'unit': '°C',
        'reference': 'AnnexA.2(2)',
        'description': r"Minimum shade air temperature with an annual probability of being exceeded p (equivalent to a mean return period of 1/p)",
        'required': [r'T_{min}', r'k_{3}', r'k_{4}', r'p'],
        'type': 'number',
    },
    {
        'id': 'G1_NODE_8',
        'standard': 'EN1991-1-5',
        'title': 'Maximum shade air temperature with exceedance probability p',
        'latex_symbol': r'T_{max,p}',
        'latex_equation': r'T_{max,p} = T_{max} \cdot \left( k_{1} - k_{2} \cdot \ln \left[ - \ln \left( 1 - p \right) \right] \right)',
        'unit': '°C',
        'reference': 'AnnexA.2(2)',
        'description': r"Maximum shade air temperature with an annual probability of being exceeded p (equivalent to a mean return period of 1/p)",
        'required': [r'T_{max}', r'k_{1}', r'k_{2}', r'p'],
        'type': 'number',
    },
    {
        'id': 'G1_NODE_9',
        'standard': 'EN1991-1-5',
        'title': 'Coefficient for maximum shade air temperature',
        'latex_symbol': r'k_{1}',
        'unit': '',
        'reference': 'AnnexA.2(2)',
        'description': r"Coefficient for calculation of maximum (minimum) shade air temperature with an annual probability of being exceeded, p, other than 0.02",
        'type': 'number',
        'default': 0.781,
        'const': True,
    },
    {
        'id': 'G1_NODE_10',
        'standard': 'EN1991-1-5',
        'title': 'Scaling factor for maximum shade air temperature',
        'latex_symbol': r'k_{2}',
        'unit': '',
        'reference': 'AnnexA.2(2)',
        'description': r"Coefficient for calculation of maximum (minimum) shade air temperature with an annual probability of being exceeded, p, other than 0.02",
        'type': 'number',
        'default': 0.056,
        'const': True,
    },
    {
        'id': 'G1_NODE_11',
        'standard': 'EN1991-1-5',
        'title': 'Coefficient for minimum shade air temperature',
        'latex_symbol': r'k_{3}',
        'unit': '',
        'reference': 'AnnexA.2(2)',
        'description': r"Coefficient for calculation of maximum (minimum) shade air temperature with an annual probability of being exceeded, p, other than 0.02",
        'type': 'number',
        'default': 0.393,
        'const': True,
    },
    {
        'id': 'G1_NODE_12',
        'standard': 'EN1991-1-5',
        'title': 'Scaling factor for minimum shade air temperature',
        'latex_symbol': r'k_{4}',
        'unit': '',
        'reference': 'AnnexA.2(2)',
        'description': r"Coefficient for calculation of maximum (minimum) shade air temperature with an annual probability of being exceeded, p, other than 0.02",
        'type': 'number',
        'default': -0.156,
        'const': True,
    },
    {
        'id': 'G1_NODE_13',
        'standard': 'EN1991-1-5',
        'title': 'Minimum uniform bridge temperature component',
        'latex_symbol': r'T_{e,min}',
        'latex_equation': r'T_{e,min} = T_{min,p} - 3',
        'unit': '°C',
        'reference': '6.1.3.1(Figure6.1)',
        'description': r"Minimum uniform bridge temperature component",
        'required': [r'Deck_types', r'T_{min,p}'],
        'type': 'number',
        'table': 'formula',
    },
    {
        'id': 'G1_NODE_14',
        'standard': 'EN1991-1-5',
        'title': 'Maximum uniform bridge temperature component',
        'latex_symbol': r'T_{e,max}',
        'latex_equation': r'T_{e,max} = T_{max,p} + 16',
        'unit': '°C',
        'reference': '6.1.3.1(Figure6.1)',
        'description': r"Maximum uniform bridge temperature component",
        'required': [r'Deck_types', r'T_{max,p}'],
        'type': 'number',
        'table': 'formula',
    },
    {
        'id': 'G1_NODE_15',
        'standard': 'EN1991-1-5',
        'title': 'Maximum contraction range of a uniform bridge temperature',
        'latex_symbol': r'\Delta T_{N,con}',
        'latex_equation': r'\Delta T_{N,con} = T_{0} - T_{e,min}',
        'unit': '°C',
        'reference': '6.1.3.3(3)',
        'description': r"Maximum contraction range of uniform bridge temperature component (T0 ≥ Te.min)",
        'required': [r'T_{0}', r'T_{e,min}'],
        'type': 'number',
    },
    {
        'id': 'G1_NODE_16',
        'standard': 'EN1991-1-5',
        'title': 'Maximum expansion range of a uniform bridge temperature',
        'latex_symbol': r'\Delta T_{N,exp}',
        'latex_equation': r'\Delta T_{N,exp} = T_{e,max} - T_{0}',
        'unit': '°C',
        'reference': '6.1.3.3(3)',
        'description': r"Maximum expansion range of uniform bridge temperature component (Te.max ≥ T0)",
        'required': [r'T_{e,max}', r'T_{0}'],
        'type': 'number',
    },

    # Example: [EN1992-1-1] Guide for Calculating Geometrical Imperfection in Bridge Design
    {
        'id': 'G2_NODE_1',
        'standard': 'EN1992-1-1', # EN1992-2(5.101) 중복
        'title': 'Geometrical imperfections in bridge',
        'latex_symbol': r'\theta_{i}',
        'latex_equation': r'\theta_{i} = \theta_{0} \times \alpha_{h}',
        'unit': 'rad',
        'reference': '5.101',
        'description': r"Inclination caused by geometrical imperfections refers to the initial deviation in the structure's alignment due to fabrication or construction errors. It affects the stability of structural members like girders, piers, and arches, and must be accounted for in design to ensure the bridge's overall stability under load.",
        'required': [r'\theta_{0}', r'\alpha_{h}'],
        'type': 'number',
    },
    {
        'id': 'G2_NODE_2',
        'standard': 'EN1992-1-1',
        'title': 'Basic value of initial imperfections in bridge',
        'latex_symbol': r'\theta_{0}',
        'unit': 'rad',
        'reference': '5.2(5)',
        'description': r"The basic value of initial imperfections in bridge structures represents the initial deviation or inclination due to construction or fabrication inaccuracies. This value, often specified in national standards, is typically set as 1/200 and serves as the foundation for calculating the overall imperfections in structural members such as girders, piers, and arches.",
        'type': 'number',
        'default': 0.005,
        'const': True,
    },
    {
        'id': 'G2_NODE_3',
        'standard': 'EN1992-1-1', # EN1992-2(5.2) 중복
        'title': 'Basic value of initial imperfections in bridge',
        'latex_symbol': r'\alpha_{h}',
        'latex_equation': r'\alpha_{h} = \min \left( \frac{2}{\sqrt{l}}, 1 \right)',
        'unit': 'rad',
        'reference': '5.2',
        'description': r"The basic value of initial imperfections in bridge structures represents the initial deviation or inclination due to construction or fabrication inaccuracies. This value, often specified in national standards, is typically set as 1/200 and serves as the foundation for calculating the overall imperfections in structural members such as girders, piers, and arches.",
        'required': [r'l'],
        'type': 'number',
    },
    {
        'id': 'G2_NODE_4',
        'standard': 'EN1992-1-1',
        'title': 'Length or height of structural members in bridge design',
        'latex_symbol': r'l',
        'unit': 'm',
        'reference': '5.2(5)',
        'description': r"The length or height of structural members is used to calculate the reduction factor for geometrical imperfections, reflecting the actual size of elements like girders, piers, and arches in bridge design.",
        'type': 'number',
        'default': 0.001,
    },
    {
        'id': 'G2_NODE_5',
        'standard': 'EN1992-1-1',
        'title': 'Eccentricity due to geometrical imperfections in isolated members',
        'latex_symbol': r'e_{i}',
        'latex_equation': r'e_{i} = \theta_{i} \times \left( \frac{l_{0}}{2} \right)',
        'unit': 'm',
        'reference': '5.2(5.2)',
        'description': r"In bridge design, eccentricity due to geometrical imperfections in isolated members refers to the deviation of the structural axis, which results in additional bending moments. This provides a simplified way to account for imperfections in structural analysis, particularly in compression elements like columns and girders.",
        'required': [r'\theta_{i}', r'l_{0}'],
        'type': 'number',
    },
    {
        'id': "G2_NODE_6",
        'standard': 'EN1992-1-1',
        'title': 'Bridge deck types',
        'latex_symbol': r'buckmode',
        'unit': '',
        'reference': '5.8.3.2(Figure5.7)',
        'description': r"Buckling modes and boundary conditions describe how structural members behave under load based on their end restraints. Different conditions, such as pinned, fixed, or guided ends, affect the way a member experiences buckling.",
        'type': 'string',
        'default': 'Pinned Ends',
        'table': 'dropdown',
        'enum': [
            "Pinned Ends",
            "Free - Fixed Ends",
            "Pinned - Fixed Ends",
            "Fixed Ends",
            "Guided - Fixed Ends",
        ],
    },
    {
        'id': 'G2_NODE_7',
        'standard': 'EN1992-1-1',
        'title': 'Effective length of isolated members',
        'latex_symbol': r'l_{0}',
        'latex_equation': r'l_{0} = l',
        'unit': 'm',
        'reference': '5.8.3.2(Figure5.7)',
        'description': r"The effective length of isolated members refers to the length used in buckling analysis to represent the portion of a structural member that is free to buckle under load. This length depends on the boundary conditions, such as pinned, fixed, or free ends. It is essential for predicting when and how isolated members, like girders or columns, will buckle.",
        'required': [r'buckmode', r'l'],
        'type': 'number',
        'table': 'formula',
    },
    {
        'id': 'G2_NODE_8',
        'standard': 'EN1992-1-1',
        'title': 'Effective length of isolated members',
        'latex_symbol': r'H_{i}',
        'latex_equation': r'H_{i} = N \times e_{i} \times \left( \frac{4}{l} \right)',
        'unit': 'kN',
        'reference': '5.2(7)',
        'description': r"The transverse force due to imperfections in isolated members refers to the lateral force applied at the position that creates the maximum moment. This force accounts for geometrical imperfections such as initial deflections and is used to ensure accurate buckling analysis in structural design.",
        'required': [r'buckmode', r'l', r'e_{i}', r'N'],
        'type': 'number',
        'table': 'formula',
    },
    {
        'id': 'G2_NODE_9',
        'standard': 'EN1992-1-1',
        'title': 'Axial load in isolated members',
        'latex_symbol': r'N',
        'unit': 'kN',
        'reference': '5.2(7)',
        'description': r"Axial load in isolated members refers to the vertical or compressive force applied along the axis of a structural element. This load is crucial in analyzing buckling behavior, as it interacts with imperfections such as initial deflection or inclination, affecting the stability of the member.",
        'type': 'number',
        'default': 0.001,
    },
    
    # Example: [EN1991-1-4] Guide to Calculating Peak Velocity Pressure at Height
    {
        'id': 'G3_NODE_1',
        'standard': 'EN1991-1-4',
        'title': 'Basic wind velocity',
        'latex_symbol': r'v_{b}',
        'latex_equation': r'v_{b} = c_{dir} \times c_{season} \times v_{b,0} \times c_{prob}',
        'unit': 'm/s',
        'reference': '4.2(4.1)',
        'description': r"The basic wind velocity refers to the mean wind speed calculated over a 10-minute period at a height of 10 meters above the ground in terrain with minimal vegetation and few obstacles. It takes into account factors like wind direction and the time of year and is adjusted using directional and seasonal factors.",
        'required': [r'c_{dir}', r'c_{season}', r'v_{b,0}', r'c_{prob}'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_2',
        'standard': 'EN1991-1-4',
        'title': 'Fundamental wind velocity',
        'latex_symbol': r'v_{b,0}',
        'unit': 'm/s',
        'reference': '4.2(1)P',
        'description': r"The fundamental wind velocity is the baseline 10-minute mean wind speed measured at a height of 10 meters above open terrain with minimal vegetation and few obstacles. It is the starting point for calculating the basic wind velocity, before applying any adjustments for wind direction or season. The value is typically provided in national standards.",
        'type': 'number',
        'default': 0.001,
    },
    {
        'id': 'G3_NODE_2',
        'standard': 'EN1991-1-4',
        'title': 'Fundamental wind velocity',
        'latex_symbol': r'v_{b,0}',
        'unit': 'm/s',
        'reference': '4.2(1)P',
        'description': r"The fundamental wind velocity is the baseline 10-minute mean wind speed measured at a height of 10 meters above open terrain with minimal vegetation and few obstacles. It is the starting point for calculating the basic wind velocity, before applying any adjustments for wind direction or season. The value is typically provided in national standards.",
        'type': 'number',
        'default': 0.001,
    },
    {
        'id': 'G3_NODE_3',
        'standard': 'EN1991-1-4',
        'title': 'Wind directional adjustment factor',
        'latex_symbol': r'c_{dir}',
        'unit': '',
        'reference': '4.2(2)P',
        'description': r"The directional factor is a coefficient that adjusts the basic wind velocity based on the wind direction. It accounts for the variability in wind speeds depending on which direction the wind is coming from. The value of this factor may vary depending on national standards, but a common recommended value is 1.0.",
        'type': 'number',
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G3_NODE_4',
        'standard': 'EN1991-1-4',
        'title': 'Wind probability adjustment factor',
        'latex_symbol': r'c_{season}',
        'unit': '',
        'reference': '4.2(2)P',
        'description': r"The seasonal adjustment factor accounts for changes in wind speed due to seasonal variations. It modifies the basic wind velocity to reflect different wind conditions throughout the year. The recommended value is often 1.0, indicating no adjustment for seasons.",
        'type': 'number',
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G3_NODE_5',
        'standard': 'EN1991-1-4',
        'title': 'Seasonal Adjustment Factor',
        'latex_symbol': r'c_{prob}',
        'latex_equation': r'c_{prob} = \left( \frac{ \left(1 - K \times \ln \left( -\ln \left(1 - p \right) \right) \right) }{ 1 - K \times \ln \left(-\ln \left( 0.98 \right) \right) } \right)^{n}',
        'unit': '',
        'reference': '4.2(4.2)',
        'description': r"The wind probability adjustment factor modifies the basic wind velocity based on the likelihood of extreme wind speeds occurring within a given year. It adjusts the wind speed to account for the probability of these events, using specific statistical parameters related to the distribution of extreme values. These parameters are typically provided in national standards.",
        'required': [r'K', r'n', r'p'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_6',
        'standard': 'EN1991-1-4',
        'title': 'Mean return period',
        'latex_symbol': r'p_{years}', # TODO : 중복 (G1_NODE_3)
        'unit': 'years',
        'reference': '3.4(1)',
        'description': r"The mean return period is the average time interval over which a specific wind event (like a wind speed) is expected to be exceeded once. A mean return period of 50 years indicates that the wind speed is likely to be exceeded once every 50 years on average.",
        'type': 'number',
        'default': 50,
    },
    {
        'id': 'G3_NODE_7',
        'standard': 'EN1991-1-4',
        'title': 'Annual exceedance probability',
        'latex_symbol': r'p', # TODO : 중복 (G1_NODE_4)
        'latex_equation': r'p = \frac{1}{(p_{years})}',
        'unit': '',
        'reference': '3.4(1)',
        'description': r"This refers to the likelihood that a specific wind speed will be exceeded in any given year. An annual probability of exceedence of 0.02 means there is a 2% chance that the wind speed will exceed this value in one year.",
        'required': [r'p_{years}'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_8',
        'standard': 'EN1991-1-4',
        'title': 'Wind speed shape parameter for extreme events',
        'latex_symbol': r'K',
        'unit': '',
        'reference': '4.2(2)P',
        'description': r"The wind speed shape parameter for extreme events is a statistical coefficient that characterizes the distribution of extreme wind speeds. It influences the adjustment of the probability factor, reflecting the variability and likelihood of extreme wind conditions. This parameter is typically determined based on the variation in wind speed data and is provided in national standards.",
        'type': 'number',
        'default': 0.2,
        'const': True,
    },
    {
        'id': 'G3_NODE_9',
        'standard': 'EN1991-1-4',
        'title': 'Wind speed shape parameter for extreme events',
        'latex_symbol': r'n',
        'unit': '',
        'reference': '4.2(2)P',
        'description': r"The exponent for wind probability adjustment is a factor used to modify the probability factor in wind speed calculations. It determines the degree of influence that the probability of extreme wind events has on the adjusted wind velocity.",
        'type': 'number',
        'default': 0.5,
        'const': True,
    },
    {
        'id': 'G3_NODE_10',
        'standard': 'EN1991-1-4',
        'title': 'Reference height above ground',
        'latex_symbol': r'z_{e}',
        'unit': 'm',
        'reference': '5.1(Table5.1)',
        'description': r"The reference height is the maximum height above ground for the section being considered. This height is used in wind load calculations to determine the peak wind velocity and other wind-related forces acting on the structure.",
        'type': 'number',
        'default': 0.001,
    },
    {
        'id': 'G3_NODE_11',
        'standard': 'EN1991-1-4',
        'title': 'Height above ground level',
        'latex_symbol': r'z',
        'latex_equation': r'z = \min \left(z_{e}, z_{max} \right)',
        'unit': 'm',
        'reference': '4.3.1(1)',
        'description': r"The height above ground level refers to the vertical distance from the ground where wind speed measurements or calculations are taken. It is a key parameter in determining wind velocity, as wind speed varies with height due to changes in terrain roughness and other factors.",
        'required': [r'z_{e}', r'z_{max}'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_12',
        'standard': 'EN1991-1-4',
        'title': 'Mean wind velocity at height',
        'latex_symbol': r'v_{m(z)}',
        'latex_equation': r'v_{m(z)} = c_{r}(z) \times c_{o}(z) \times v_{b}',
        'unit': 'm/s',
        'reference': '4.3.1(4.3)',
        'description': r"The mean wind velocity at height refers to the average wind speed at a specific height above the ground. This value is influenced by factors such as the roughness of the terrain, the topography (orography), and the basic wind velocity. It is calculated using the roughness and orography adjustment factors, which modify the wind speed to reflect the effects of the surrounding environment at different heights.",
        'required': [r'c_{r}(z)', r'c_{o}(z)', r'v_{b}'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_13',
        'standard': 'EN1991-1-4',
        'title': 'Orography adjustment factor for mean wind speed',
        'latex_symbol': r'c_{o}(z)',
        'unit': '',
        'reference': '4.2(2)P',
        'description': r"The orography adjustment factor for mean wind speed takes into account the effects of terrain features such as hills or valleys on wind speed at various heights. It adjusts the mean wind speed based on these topographical variations. Typically, the value is set to 1.0 unless specified otherwise in national standards, indicating no adjustment for orography.",
        'type': 'number',
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G3_NODE_14',
        'standard': 'EN1991-1-4',
        'title': 'Terrain roughness factor for mean wind velocity',
        'latex_symbol': r'c_{r}(z)',
        'latex_equation': r'c_{r(z_{min})} = k_{r} \times \ln \left(\frac {z_{min}}{z_{0}} \right)',
        'unit': 'm/s',
        'reference': '4.3.2(4.4)',
        'description': r"The terrain roughness factor adjusts the mean wind velocity to account for changes in wind speed caused by the roughness of the terrain and the height above the ground. It is determined using a logarithmic formula and varies based on the roughness of the terrain and specific height.",
        'required': [r'z', r'z_{min}', r'z_{0}', r'k_{r}'],
        'type': 'number',
        'table': 'formula',
    },
    {
        'id': "G3_NODE_15",
        'standard': 'EN1991-1-4',
        'title': 'Terrain category selection',
        'latex_symbol': r'terrain',
        'unit': '',
        'reference': '4.3.2(Table4.1)',
        'description': r"This section allows the user to choose the appropriate terrain category based on the characteristics of the surrounding area. Terrain categories are used to define the roughness of the ground, which affects wind speed calculations. Options typically range from open terrain with low vegetation to urban environments with numerous obstacles.",
        'type': 'string',
        'default': '0',
        'table': 'dropdown',
        'enum': [
            "0",
            "I",
            "II",
            "III",
            "IV",
        ],
    },
    {
        'id': "G3_NODE_16",
        'standard': 'EN1991-1-4',
        'title': 'Roughness length for terrain',
        'latex_symbol': r'z_{0}',
        'unit': 'm',
        'reference': '4.3.2(Table4.1)',
        'description': r"The roughness length is a parameter that represents the height above ground level where the wind speed theoretically becomes zero due to surface roughness. It is used in the calculation of the terrain roughness factor and depends on the type of terrain. The value of the roughness length varies according to different terrain categories.",
        'type': 'number',
        'default': 0.003,
        'table' : 'formula',
    },
    {
        'id': "G3_NODE_17",
        'standard': 'EN1991-1-4',
        'title': 'Minimum height for roughness factor calculation',
        'latex_symbol': r'z_{min}',
        'unit': 'm',
        'reference': '4.3.2(Table4.1)',
        'description': r"The minimum height is the lowest height above the ground at which the roughness factor is applicable. Below this height, the roughness factor remains constant. The value of the minimum height is determined by the terrain category and is used in the calculation of wind speed adjustments.",
        'type': 'number',
        'default': 1,
        'table' : 'formula',
    },
    {
        'id': 'G3_NODE_18',
        'standard': 'EN1991-1-4',
        'title': 'Terrain factor for roughness calculation',
        'latex_symbol': r'k_{r}',
        'latex_equation': r'k_{r} = 0.19 \times \left( \frac{z_{0}}{z_{0,II}} \right) ^ {0.07}',
        'unit': 'm/s',
        'reference': '4.3.2(4.5)',
        'description': r"The terrain factor is a coefficient used in the calculation of the roughness factor, reflecting the influence of the terrain's roughness length. It depends on the ratio between the roughness length of the terrain and the standard roughness length for a reference terrain (terrain category II). This factor helps adjust wind speed based on different terrain conditions.",
        'required': [r'z_{0}', r'z_{0,II}'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_19',
        'standard': 'EN1991-1-4',
        'title': 'Reference roughness length for terrain category II',
        'latex_symbol': r'z_{0,II}',
        'unit': 'm',
        'reference': '4.3.2(Table4.1)',
        'description': r"The reference roughness length for terrain category II is a standard value used as a baseline for calculating the terrain factor in wind speed assessments. It represents the roughness length for open terrain with low vegetation, typically set at 0.05 meters. This value is used to compare other terrain roughness lengths and adjust the wind speed accordingly.",
        'type': 'number',
        'default': 0.05,
        'const': True,
    },
    {
        'id': 'G3_NODE_20',
        'standard': 'EN1991-1-4',
        'title': 'Maximum height for roughness factor calculation',
        'latex_symbol': r'z_{max}',
        'unit': 'm',
        'reference': '4.3.2(1)',
        'description': r"The maximum height is the highest altitude above ground where the roughness factor is considered in wind speed calculations. Above this height, the roughness factor does not apply. For most cases, the maximum height is set at 200 meters, unless otherwise specified.",
        'type': 'number',
        'default': 200,
        'const': True,
    },
    {
        'id': 'G3_NODE_21',
        'standard': 'EN1991-1-4',
        'title': 'Standard deviation of wind turbulence',
        'latex_symbol': r'\sigma_{v}',
        'latex_equation': r'\sigma_{v} = k_{r} \times v_{b} \times k_{l}',
        'unit': 'kN/m^2',
        'reference': '4.5(4.8)',
        'description': r"The peak velocity pressure at height accounts for both the mean wind velocity and short-term wind fluctuations at a specific height above the ground. It is calculated using the air density, mean wind velocity, and an exposure factor, which reflects the terrain and other local conditions. The peak velocity pressure helps in determining the wind loads on structures.",
        'required': [r'k_{r}', r'v_{b}', r'k_{l}'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_22',
        'standard': 'EN1991-1-4',
        'title': 'Turbulence factor',
        'latex_symbol': r'k_{l}',
        'unit': 'm',
        'reference': '4.4(1)',
        'description': r"The turbulence factor is a coefficient used in the calculation of wind turbulence intensity. It accounts for the variability of turbulence in different wind conditions and terrains. The value of the turbulence factor may be specified in national standards, with a commonly recommended value being 1.0. This factor adjusts the standard deviation of wind turbulence in relation to the basic wind velocity.",
        'type': 'number',
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G3_NODE_23',
        'standard': 'EN1991-1-4',
        'title': 'Turbulence intensity at height',
        'latex_symbol': r'I_{v}(z)',
        'latex_equation': r'I_{v}(z) = \frac{k_{l}}{c_{o}(z) \times \ln \left( \frac{z_{min}}{z_{0}} \right)}',
        'unit': 'm/s',
        'reference': '4.4(4.7)',
        'description': r"Turbulence intensity at height is the ratio of the standard deviation of wind turbulence to the mean wind velocity at a specific height. It measures the level of wind speed fluctuations due to turbulence. The value of turbulence intensity depends on the terrain roughness and height above ground. It is calculated using a formula that incorporates factors such as the roughness length and orography.",
        'required': [r'k_{l}', r'c_{o}(z)', r'z', r'z_{min}', r'z_{0}'],
        'type': 'number',
        'table': 'formula',
    },
    {
        'id': 'G3_NODE_24',
        'standard': 'EN1991-1-4',
        'title': 'Peak velocity pressure at height',
        'latex_symbol': r'q_{p(z)}',
        'latex_equation': r'q_{p(z)} = \left[ 1 + 7 \times I_{v}(z)  \right] \cdot \frac{1}{2} \times \rho \times v_{m(z)}^{2} \times 10^{-3}',
        'unit': 'kN/m^2',
        'reference': '4.5(4.8)',
        'description': r"The peak velocity pressure at height accounts for both the mean wind velocity and short-term wind fluctuations at a specific height above the ground. It is calculated using the air density, mean wind velocity, and an exposure factor, which reflects the terrain and other local conditions. The peak velocity pressure helps in determining the wind loads on structures.",
        'required': [r'I_{v}(z)', r'\rho', r'v_{m(z)}'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_25',
        'standard': 'EN1991-1-4',
        'title': 'Air density during wind storms',
        'latex_symbol': r'\rho',
        'unit': 'kg/m^3',
        'reference': '4.5(1)',
        'description': r"Air density during wind storms is a crucial factor in wind load calculations, as it depends on altitude, temperature, and barometric pressure expected in the region during extreme wind events. This parameter affects the calculation of peak velocity pressure and can vary with changing atmospheric conditions.",
        'type': 'number',
        'default': 1.25,
        'const': True,
    },
    {
        'id': 'G3_NODE_26',
        'standard': 'EN1991-1-4',
        'title': 'Exposure factor at heigh',
        'latex_symbol': r'c_{e(z)}',
        'latex_equation': r'c_{e(z)} = \frac{q_{p(z)}}{q_{b}}',
        'unit': '',
        'reference': '4.5(4.9)',
        'description': r"The exposure factor at height adjusts the peak velocity pressure based on the height above ground and the surrounding terrain conditions. It reflects the influence of terrain roughness and exposure to wind, varying with the type of terrain and height. This factor helps modify the peak wind pressure to better represent real-world conditions at different elevations.",
        'required': [r'q_{p(z)}', r'q_{b}'],
        'type': 'number',
    },
    {
        'id': 'G3_NODE_27',
        'standard': 'EN1991-1-4',
        'title': 'Exposure factor at heigh',
        'latex_symbol': r'q_{b}',
        'latex_equation': r'q_{b} = \left( \frac{1}{2} \times \rho \times v_{b}^{2} \right) \times 10^{-3}',
        'unit': 'kN/m^2',
        'reference': '4.5(4.10)',
        'description': r"The basic velocity pressure represents the wind pressure at a given location based on the basic wind velocity. It is calculated using air density and the square of the basic wind velocity. This value serves as a reference for further adjustments when calculating the peak velocity pressure at different heights or under different terrain conditions.",
        'required': [r'\rho', r'v_{b}'],
        'type': 'number',
    },

    # Example: [EN1991-1-1] Nominal Density for Construction Materials
    {
        'id': 'G4_NODE_1',
        'standard': 'EN1991-1-1',
        'title': 'Density Type',
        'latex_symbol': r'densitytype',
        'unit': '',
        'reference': 'AnnexA',
        'description': r"'Density Type' clearly categorizes the different forms of density, such as 'Mass Density' and 'Weight Density.' This title helps distinguish between how density is expressed, whether as mass per unit volume (kg/m³) or weight per unit volume (kN/m³), ensuring clarity in the table.",
        'type': 'string',
        'default': "Weight density (kN/m^3)",
        'table': 'dropdown',
        'enum': [
            "Weight density (kN/m^3)",
            "Mass density (kg/m^3)",
        ]
    },
    {
        'id': 'G4_NODE_2',
        'standard': 'EN1991-1-1',
        'title': 'Gravity acceleration',
        'latex_symbol': r'g',
        'unit': 'm/s^2',
        'reference': 'AnnexA',
        'description': r"Gravity acceleration is applied to convert weight density to mass density. The value of gravity acceleration follows Eurocode standards. Detailed information can be found in EN 1998-1 Section 1.7.",
        'type': 'number',
        'default': '9.81',
        'const': True,
    },
    {
        'id': 'G4_NODE_3',
        'standard': 'EN1991-1-1',
        'title': 'kN to kg unit converter',
        'latex_symbol': r'kntokg',
        'latex_equation': r'kntokg = 1',
        'unit': 'm/s^2',
        'reference': '',
        'description': r"This logic converts between kilonewtons (kN) and kilograms (kg) by applying gravitational acceleration, offering an efficient way to handle force and mass conversions.",
        'required': [r'densitytype', r'g'],
        'type': 'number',
        'table': 'formula',
    },
    {
        'id': 'G4_NODE_4',
        'standard': 'EN1991-1-1',
        'title': 'Density of concrete material',
        'latex_symbol': r'densityconcrete',
        'unit': '',
        'reference': 'AnnexA(Table A.6)',
        'description': r"This menu provides the density corresponding to the selected type of concrete, allowing users to easily find the appropriate density for normal weight, reinforced, or pre-stressed concrete.",
        'required': [r'kntokg'],
        'type': 'object', # TODO : Table? Object?
        'table': 'result',
    },
    {
        'id': 'G4_NODE_5',
        'standard': 'EN1991-1-1',
        'title': 'Pavement of road bridges',
        'latex_symbol': r'paveroad',
        'unit': '',
        'reference': 'AnnexA(Table A.6)',
        'description': r"This section provides the density values for different types of asphalt used in road bridge pavements, including Guss Asphalt for waterproofing and bridge decks, Mastic Asphalt for dense and smooth surfaces, and Hot Rolled Asphalt (HRA) for durable, wear-resistant applications.",
        'required': [r'kntokg'],
        'type': 'object', # TODO : Table? Object?
        'table': 'result',
    },
    {
        'id': 'G4_NODE_6',
        'standard': 'EN1991-1-1',
        'title': 'Pavement of rail bridges',
        'latex_symbol': r'paverail',
        'unit': '',
        'reference': 'AnnexA(Table A.6)',
        'description': r"This section provides the density values for materials used in rail bridge pavements, including the concrete protective layer, normal ballast (e.g., granite, gneiss), and basaltic ballast. These materials are essential for ensuring the durability and stability of the bridge surface.",
        'required': [r'kntokg'],
        'type': 'object', # TODO : Table? Object?
        'table': 'result',
    },
    {
        'id': 'G4_NODE_7',
        'standard': 'EN1991-1-1',
        'title': 'Infill for bridges',
        'latex_symbol': r'infill',
        'unit': 'm/s^2',
        'reference': 'AnnexA(Table A.6)',
        'description': r"This section lists the density values for various infill materials used in bridge construction, including ballast, gravel (loose), hardcore, crushed slag, packed stone rubble, and puddle clay. These materials are essential for providing stability, drainage, and support in bridge structures.",
        'required': [r'kntokg'],
        'type': 'object', # TODO : Table? Object?
        'table': 'result',
    },
    {
        'id': 'G4_NODE_8',
        'standard': 'EN1991-1-1',
        'title': 'Structures with ballasted bed',
        'latex_symbol': r'strutwithballasted',
        'unit': '',
        'reference': 'AnnexA(Table A.6)',
        'description': r"This section provides the density values for various track components used in structures with ballasted beds, including 2 rails UIC 60, prestressed concrete sleepers with track fastenings, concrete sleepers with metal angle braces, and timber sleepers with track fastenings.",
        'required': [r'kntokg'],
        'type': 'object', # TODO : Table? Object?
        'table': 'result',
    },
    {
        'id': 'G4_NODE_9',
        'standard': 'EN1991-1-1',
        'title': 'Structures without ballasted bed',
        'latex_symbol': r'strutwithoutballasted',
        'unit': '',
        'reference': 'AnnexA(Table A.6)',
        'description': r"This section provides the density values for components used in structures without a ballasted bed, including 2 rails UIC 60 with track fastenings, bridge beams, and guard rails. ",
        'required': [r'kntokg'],
        'type': 'object', # TODO : Table? Object?
        'table': 'result',
    },
]

data_table = [
    {
        "id": "G1_NODE_1",
        "data": [
            {
                'condition': r'steel box girder',
                'value': 'Type1',
            },
            {
                'condition': r'steel truss or plate girder',
                'value': 'Type1',
            },
            {
                'condition': r'composite deck',
                'value': 'Type2',
            },
            {
                'condition': r'concrete slab',
                'value': 'Type3',
            },
            {
                'condition': r'concrete beam',
                'value': 'Type3',
            },
            {
                'condition': r'concrete box girder',
                'value': 'Type3',
            }
        ],
    },
    {
        "id": "G1_NODE_13",
        "data": [
            {
                'condition': r'Deck_types = Type1',
                'value': r'T_{e,min} = T_{min,p} - 3',
            },
            {
                'condition': r'Deck_types = Type2',
                'value': r'T_{e,min} = T_{min,p} + 4',
            },
            {
                'condition': r'Deck_types = Type3',
                'value': r'T_{e,min} = T_{min,p} + 8',
            },
        ],
    },
    {
        "id": "G1_NODE_14",
        "data": [
            {
                'condition': r'Deck_types = Type1',
                'value': r'T_{e,max} = T_{max,p} - 16',
            },
            {
                'condition': r'Deck_types = Type2',
                'value': r'T_{e,max} = T_{max,p} + 4',
            },
            {
                'condition': r'Deck_types = Type3',
                'value': r'T_{e,max} = T_{max,p} + 2',
            },
        ],
    },
    {
        "id": "G2_NODE_6",
        "data": [
            {
                'condition': r'Pinned Ends',
                'value': 'PinnedEnds',
            },
            {
                'condition': r'Free - Fixed Ends',
                'value': 'Free-FixedEnds',
            },
            {
                'condition': r'Pinned - Fixed Ends',
                'value': 'Pinned-FixedEnds',
            },
            {
                'condition': r'Fixed Ends',
                'value': 'FixedEnds',
            },
            {
                'condition': r'Guided - Fixed Ends',
                'value': 'Guided-FixedEnds',
            },
        ],
    },
    {
        "id": "G2_NODE_7",
        "data": [
            {
                'condition': r'buckmode = PinnedEnds',
                'value': r'l_{0} = l',
            },
            {
                'condition': r'buckmode = Free-FixedEnds',
                'value': r'l_{0} = 2 \times l',
            },
            {
                'condition': r'buckmode = Pinned-FixedEnds',
                'value': r'l_{0} = 0.7 \times l',
            },
            {
                'condition': r'buckmode = FixedEnds',
                'value': r'l_{0} = 0.5 \times l',
            },
            {
                'condition': r'buckmode = Guided-FixedEnds',
                'value': r'l_{0} = l',
            },
        ],
    },
    {
        "id": "G2_NODE_8",
        "data": [
            {
                'condition': r'buckmode = PinnedEnds',
                'value': r'H_{i} = N \times e_{i} \times \left( \frac{4}{l} \right)',
            },
            {
                'condition': r'buckmode = Free-FixedEnds',
                'value': r'H_{i} = N \times e_{i} \times \left( \frac{1}{l} \right)',
            },
            {
                'condition': r'buckmode = Pinned-FixedEnds',
                'value': r'H_{i} = N \times e_{i} \times \left( \frac{16}{3 \times l} \right)',
            },
            {
                'condition': r'buckmode = FixedEnds',
                'value': r'H_{i} = N \times e_{i} \times \left( \frac{8}{l} \right)',
            },
            {
                'condition': r'buckmode = Guided-FixedEnds',
                'value': r'H_{i} = N \times e_{i} \times \left( \frac{2}{l} \right)',
            },
        ],
    },
    {
        "id": "G3_NODE_14",
        "data": [
            {
                'condition': r'z <= z_{min}',
                'value': r'c_{r}(z_{min}) = k_{r} \times \ln \left(\frac {z_{min}}{z_{0}} \right)',
            },
            {
                'condition': r'z > z_{min}',
                'value': r'c_{r}(z) = k_{r} \times \ln \left(\frac {z}{z_{0}} \right)',
            },
        ],
    },
    {
        "id": "G3_NODE_15",
        "data": [
            {
                'condition': r'0',
                'value': '0',
            },
            {
                'condition': r'I',
                'value': 'I',
            },
            {
                'condition': r'II',
                'value': 'II',
            },
            {
                'condition': r'III',
                'value': 'III',
            },
            {
                'condition': r'IV',
                'value': 'IV',
            },
        ],
    },
    {
        "id": "G3_NODE_16",
        "data": [
            {
                'condition': r'terrain = 0',
                'value': 0.003,
            },
            {
                'condition': r'terrain = I',
                'value': 0.010,
            },
            {
                'condition': r'terrain = II',
                'value': 0.050,
            },
            {
                'condition': r'terrain = III',
                'value': 0.300,
            },
            {
                'condition': r'terrain = IV',
                'value': 1.000,
            },
        ],
    },
    {
        "id": "G3_NODE_17",
        "data": [
            {
                'condition': r'terrain = 0',
                'value': 1,
            },
            {
                'condition': r'terrain = I',
                'value': 1,
            },
            {
                'condition': r'terrain = II',
                'value': 2,
            },
            {
                'condition': r'terrain = III',
                'value': 5,
            },
            {
                'condition': r'terrain = IV',
                'value': 10,
            },
        ],
    },
    {
        "id": "G3_NODE_23",
        "data": [
            {
                'condition': r'z < z_{min}',
                'value': r'I_{v}(z) = \frac{k_{l}}{c_{o}(z) \times \ln \left( \frac{z_{min}}{z_{0}} \right)}',
            },
            {
                'condition': r'z >= z_{min}',
                'value': r'I_{v}(z) = \frac{k_{l}}{c_{o}(z) \times \ln \left( \frac{z}{z_{0}} \right)}',
            },
        ],
    },
    {
        "id": "G4_NODE_1",
        "data": [
            {
                'condition': r'Weight density (kN/m^3)',
                'value': '0',
            },
            {
                'condition': r'Mass density (kg/m^3)',
                'value': '1',
            },
        ],
    },
    {
        "id": "G4_NODE_3",
        "data": [
            {
                'condition': r'densitytype = 0',
                'value': r'kntokg = 1',
            },
            {
                'condition': r'densitytype = 1',
                'value': r'kntokg = 1000/g',
            },
        ],
    },
    {
        "id": "G4_NODE_4",
        "data": [
            {
                'Concrete Type': r'Normal weight concrete',
                'Density': r'24 \times kntokg',
            },
            {
                'Concrete Type': r'Normal weight reinforced concrete',
                'value': r'25 \times kntokg',
            },
            {
                'Concrete Type': r'Normal weight pre-stressed concrete',
                'value': r'25 \times kntokg',
            },
            {
                'Concrete Type': r'(unhardened)Normal weight concrete',
                'value': r'25 \times kntokg',
            },
            {
                'Concrete Type': r'(unhardened)Normal weight reinforced concrete',
                'value': r'26 \times kntokg',
            },
            {
                'Concrete Type': r'(unhardened)Normal weight pre-stressed concrete',
                'value': r'26 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_NODE_5",
        "data": [
            {
                'Pavement Type': r'Gussasphalt and asphaltic concrete',
                'Density': r'24 \times kntokg \text{~} 25 \times kntokg',
            },
            {
                'Pavement Type': r'Mastic asphalt',
                'value': r'18 \times kntokg \text{~} 22 \times kntokg',
            },
            {
                'Pavement Type': r'Hot rolled asphalt',
                'value': r'23 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_NODE_6",
        "data": [
            {
                'Pavement Type': r'concrete protective layer',
                'Density': r'25 \times kntokg',
            },
            {
                'Pavement Type': r'normal ballast (e.g. granite, gneiss, etc.)',
                'value': r'20 \times kntokg',
            },
            {
                'Pavement Type': r'basaltic ballast',
                'value': r'26 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_NODE_7",
        "data": [
            {
                'Infill Material Type': r'sand (dry)',
                'Density': r'15 \times kntokg \text{~} 16 \times kntokg',
            },
            {
                'Infill Material Type': r'ballast, gravel (loose)',
                'value': r'15 \times kntokg \text{~} 16 \times kntokg',
            },
            {
                'Infill Material Type': r'hardcore',
                'value': r'18.5 \times kntokg \text{~} 19.5 \times kntokg',
            },
            {
                'Infill Material Type': r'crushed slag',
                'value': r'13.5 \times kntokg \text{~} 14.5 \times kntokg',
            },
            {
                'Infill Material Type': r'packed stone rubble',
                'value': r'20.5 \times kntokg \text{~} 21.5 \times kntokg',
            },
            {
                'Infill Material Type': r'puddle clay',
                'value': r'18.5 \times kntokg \text{~} 19.5 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_NODE_8",
        "data": [
            {
                'Track Components': r'2 rails UIC 60',
                'Density': r'1.2 \times kntokg',
            },
            {
                'Track Components': r'prestressed concrete sleeper with track fastenings',
                'Density': r'4.8 \times kntokg',
            },
            {
                'Track Components': r'timber sleepers with track fastenings',
                'Density': r'1.9 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_NODE_9",
        "data": [
            {
                'Track Components': r'2 rails UIC 60 with track fastenings',
                'Density': r'1.7 \times kntokg',
            },
            {
                'Track Components': r'bridge beam and guard rails',
                'Density': r'4.9 \times kntokg',
            },
        ],
    },
]